@@ -1,4 +1,5 @@
 import ast
+import dis
 import re
 import sys
 import types
@@ -9,6 +10,8 @@ from collections import defaultdict
 from collections import deque
 from datetime import date
 from datetime import datetime
+from datetime import time
+from datetime import timedelta
 from inspect import CO_VARARGS
 from inspect import CO_VARKEYWORDS
 
@@ -40,7 +43,7 @@ else:
 try:
     from types import InstanceType
 except ImportError:
-    InstanceType = object()
+    InstanceType = type(object())
 
 try:
     from re import Pattern
@@ -53,6 +56,7 @@ try:
 except ImportError:
     RegexFlag = int
 
+PY2 = sys.version_info[0] == 2
 PY3 = sys.version_info[0] == 3
 
 if PY3:
@@ -71,6 +75,7 @@ OTHER_COLORS = {
     'INTERNAL-DETAIL': Fore.WHITE,
     'SOURCE-FAILURE': Style.BRIGHT + Back.YELLOW + Fore.YELLOW,
     'SOURCE-DETAIL': Fore.WHITE,
+    'BUILTIN': Style.NORMAL + Fore.MAGENTA,
 
     'RESET': Style.RESET_ALL,
 }
@@ -105,26 +110,73 @@ if PY3:
         co_kwonlyargcount = code.co_kwonlyargcount
         kwonlyargs = co_varnames[co_argcount:co_argcount + co_kwonlyargcount]
         for arg in co_varnames[:co_argcount]:
-            yield '', arg
+            yield '', arg, arg
         co_argcount += co_kwonlyargcount
         if code.co_flags & CO_VARARGS:
-            yield '*', co_varnames[co_argcount]
+            arg = co_varnames[co_argcount]
+            yield '*', arg, arg
             co_argcount = co_argcount + 1
         for arg in kwonlyargs:
-            yield '', arg
+            yield '', arg, arg
         if code.co_flags & CO_VARKEYWORDS:
-            yield '**', co_varnames[co_argcount]
+            arg = co_varnames[co_argcount]
+            yield '**', arg, arg
 else:
-    from inspect import getargs
+    class get_arguments(object):
+        def __init__(self, code):
+            self.co_varnames = code.co_varnames
+            self.co_argcount = code.co_argcount
+            self.co_code = code.co_code
+            self.co_flags = code.co_flags
+            self.position = 0
 
-    def get_arguments(code):
-        arguments = getargs(code)
-        for arg in flatten(arguments.args):
-            yield '', arg
-        if arguments.varargs:
-            yield '*', arguments.varargs
-        if arguments.keywords:
-            yield '**', arguments.keywords
+        def _get_op_arg(self):
+            value = ord(self.co_code[self.position]) + ord(self.co_code[self.position + 1]) * 256
+            self.position += 2
+            return value
+
+        def _get_op(self):
+            op = dis.opname[ord(self.co_code[self.position])]
+            self.position += 1
+            return op
+
+        def _get_unpacked_arguments(self):
+            value = self._get_op_arg()
+            for _ in range(value):
+                op = self._get_op()
+                if op == 'STORE_FAST':
+                    value = self._get_op_arg()
+                    yield self.co_varnames[value]
+                elif op == 'UNPACK_SEQUENCE':
+                    yield list(self._get_unpacked_arguments())
+
+        def _clean_name(self, name):
+            if isinstance(name, str):
+                return name
+            elif len(name) > 1:
+                return '(%s)' % ', '.join(self._clean_name(i) for i in name)
+            else:
+                return '(%s,)' % ', '.join(self._clean_name(i) for i in name)
+
+        def __iter__(self):
+            for arg in self.co_varnames[:self.co_argcount]:
+                names = None
+                if arg.startswith('.'):
+                    while self.position + 3 < len(self.co_code):
+                        op = self._get_op()
+                        value = self._get_op_arg()
+                        if op == 'LOAD_FAST' and self.co_varnames[value] == arg:
+                            op = self._get_op()
+                            if op == 'UNPACK_SEQUENCE':
+                                names = list(self._get_unpacked_arguments())
+                                break
+                if names is None:
+                    yield '', arg, arg
+                else:
+                    yield '', arg, self._clean_name(names)
+            if self.co_flags & CO_VARKEYWORDS:
+                arg = self.co_varnames[self.co_argcount]
+                yield '**', arg, arg
 
 
 def flatten(something):
@@ -229,14 +281,22 @@ def safe_repr(obj, maxdepth=5):
             )
         else:
             return 're.compile(%s)' % safe_repr(obj.pattern)
-    elif obj_type is date:
+    elif obj_type in (date, timedelta):
         return repr(obj)
     elif obj_type is datetime:
-        return '%s(%d, %d, %d, %d, %d, %d, %d, tzinfo=%s)' % (
+        return '%s(%d, %d, %d, %d, %d, %d, %d, tzinfo=%s%s)' % (
             obj_type.__name__,
             obj.year, obj.month, obj.day,
             obj.hour, obj.minute, obj.second, obj.microsecond,
-            safe_repr(obj.tzinfo)
+            safe_repr(obj.tzinfo),
+            ', fold=%s' % safe_repr(obj.fold) if hasattr(obj, 'fold') else '',
+        )
+    elif obj_type is time:
+        return '%s(%d, %d, %d, %d, tzinfo=%s%s)' % (
+            obj_type.__name__,
+            obj.hour, obj.minute, obj.second, obj.microsecond,
+            safe_repr(obj.tzinfo),
+            ', fold=%s' % safe_repr(obj.fold) if hasattr(obj, 'fold') else '',
         )
     elif obj_type is types.MethodType:  # noqa
         self = obj.__self__
